MODULE Fog_Detection

!$Id: fog_detection.f90,v 1.21 2009/04/24 15:22:55 mpav Exp $

!---------------------------------------------------------------
! Module use statements
!---------------------------------------------------------------
      
USE ALGORITHM_MODULE_USAGE

IMPLICIT NONE

PRIVATE

PUBLIC::Fog_Detection_Main
PRIVATE :: Read_Fog_LUT

CHARACTER(Len=*), PARAMETER::LUT_Filename = "fog_training_histograms_Nov13_2008_20:58:18.hdf"

INTEGER(KIND=Int4), PRIVATE, SAVE :: Refl_Chn2_Nbins_LUT
REAL(KIND=Real4), DIMENSION(:), ALLOCATABLE, PRIVATE, SAVE :: Refl_Chn2_Bin_Start_LUT
REAL(KIND=Real4), DIMENSION(:), ALLOCATABLE, PRIVATE, SAVE :: Refl_Chn2_Bin_End_LUT

INTEGER(KIND=Int4), PRIVATE, SAVE :: Emiss_Chn14_Nbins_LUT
REAL(KIND=Real4), DIMENSION(:), ALLOCATABLE, PRIVATE, SAVE :: Emiss_Chn14_Bin_Start_LUT
REAL(KIND=Real4), DIMENSION(:), ALLOCATABLE, PRIVATE, SAVE :: Emiss_Chn14_Bin_End_LUT

INTEGER(KIND=Int4), PRIVATE, SAVE :: Emiss_Chn7_Nbins_LUT
REAL(KIND=Real4), DIMENSION(:), ALLOCATABLE, PRIVATE, SAVE :: Emiss_Chn7_Bin_Start_LUT
REAL(KIND=Real4), DIMENSION(:), ALLOCATABLE, PRIVATE, SAVE :: Emiss_Chn7_Bin_End_LUT

INTEGER(KIND=Int4), PRIVATE, SAVE :: Tsfc_Bias_Nbins_LUT
REAL(KIND=Real4), PRIVATE, SAVE :: Tsfc_Bias_Min_LUT
REAL(KIND=Real4), PRIVATE, SAVE :: Tsfc_Bias_Max_LUT
REAL(KIND=Real4), PRIVATE, SAVE :: Tsfc_Bias_Increment_LUT

INTEGER(KIND=Int4), PRIVATE, SAVE :: BT_Chn14_Diff_Nbins_LUT
REAL(KIND=Real4), PRIVATE, SAVE :: BT_Chn14_Diff_Min_LUT
REAL(KIND=Real4), PRIVATE, SAVE :: BT_Chn14_Diff_Max_LUT
REAL(KIND=Real4), PRIVATE, SAVE :: BT_Chn14_Diff_Increment_LUT

INTEGER(KIND=Int4), PRIVATE, SAVE :: Refl_Chn2_StdDev_Nbins_LUT
REAL(KIND=Real4), PRIVATE, SAVE :: Refl_Chn2_StdDev_Min_LUT
REAL(KIND=Real4), PRIVATE, SAVE :: Refl_Chn2_StdDev_Max_LUT
REAL(KIND=Real4), PRIVATE, SAVE :: Refl_Chn2_StdDev_Increment_LUT

INTEGER(KIND=Int4), PRIVATE, SAVE :: BT_Chn14_StdDev_Nbins_LUT
REAL(KIND=Real4), PRIVATE, SAVE :: BT_Chn14_StdDev_Min_LUT
REAL(KIND=Real4), PRIVATE, SAVE :: BT_Chn14_StdDev_Max_LUT
REAL(KIND=Real4), PRIVATE, SAVE :: BT_Chn14_StdDev_Increment_LUT

REAL(KIND=Real8), DIMENSION(:,:,:), ALLOCATABLE, PRIVATE, SAVE :: Fog_Probability_Refl_Chn2_Tsfc_Bias_Water
REAL(KIND=Real8), DIMENSION(:,:,:), ALLOCATABLE, PRIVATE, SAVE :: Fog_Probability_Refl_Chn2_Tsfc_Bias_Land
REAL(KIND=Real8), DIMENSION(:,:,:), ALLOCATABLE, PRIVATE, SAVE :: Fog_Probability_Refl_Chn2_BT_Chn14_Diff_Land

REAL(KIND=Real8), DIMENSION(:,:,:), ALLOCATABLE, PRIVATE, SAVE :: Fog_Probability_Emiss_Chn7_Tsfc_Bias_Water
REAL(KIND=Real8), DIMENSION(:,:,:), ALLOCATABLE, PRIVATE, SAVE :: Fog_Probability_Emiss_Chn7_Tsfc_Bias_Land
REAL(KIND=Real8), DIMENSION(:,:,:), ALLOCATABLE, PRIVATE, SAVE :: Fog_Probability_Emiss_Chn7_BT_Chn14_Diff_Land

REAL(KIND=Real8), DIMENSION(:,:,:), ALLOCATABLE, PRIVATE, SAVE :: Fog_Probability_Emiss_Chn14_Tsfc_Bias_Water
REAL(KIND=Real8), DIMENSION(:,:,:), ALLOCATABLE, PRIVATE, SAVE :: Fog_Probability_Emiss_Chn14_Tsfc_Bias_Land
REAL(KIND=Real8), DIMENSION(:,:,:), ALLOCATABLE, PRIVATE, SAVE :: Fog_Probability_Emiss_Chn14_BT_Chn14_Diff_Land

CONTAINS

SUBROUTINE Fog_Detection_Main(Ialgo)

  !-GEOCAT algorithm index
  INTEGER(KIND=Int4), INTENT(IN)::Ialgo
  
  !---------------------------------------------------------------
  ! Begin local variable declaration
  !---------------------------------------------------------------
  
  CHARACTER(LEN=*), PARAMETER::Routine_Name = &
    "FOG_DETECTION:fog_detection_main"
  
  INTEGER(KIND=INT4), SAVE :: EMS7_DELTA_PROB
  INTEGER(KIND=INT4), SAVE :: EMS7_DELTA_DEPTH
  
  !-scan line element number
  INTEGER(KIND=Int4)::Ielem 
  
  !-scan line number
  INTEGER(KIND=Int4)::Iline
  
  !-viewing zenith angle index
  INTEGER(KIND=Int4)::Ivza
  
  !-east/west index into gridded NWP data
  INTEGER(KIND=Int4)::Xnwp
  
  !-north/south index into gridded NWP data
  INTEGER(KIND=Int4)::Ynwp
  
  !-the vertical level that corresponds to the tropopause
  INTEGER(KIND=Int4)::Itropo
  
  !-the vertical level that corresponds to the surface
  INTEGER(KIND=Int4)::Isfc
  
  !-A generic index used when interpolating between levels
  !INTEGER(KIND=int4)::ilev
  
  !-The total number of valid levels in vertical profile
  INTEGER(KIND=Int4)::Nprof
  
  INTEGER(KIND=Int4)::Astatus
  
  INTEGER(KIND=Int4) :: Refl_Chn2_LUT_Idx
  INTEGER(KIND=Int4) :: Emiss_Chn14_LUT_Idx
  INTEGER(KIND=Int4) :: Emiss_Chn7_LUT_Idx
  INTEGER(KIND=Int4) :: Tsfc_Bias_LUT_Idx
  !INTEGER(KIND=int4) :: bt14_diff_lut_index
  INTEGER(KIND=Int4) :: Refl_Chn2_StdDev_LUT_Idx
  INTEGER(KIND=Int4) :: BT_Chn14_StdDev_LUT_Idx
  
  !-The index (in the scan line element direction)
  !corresponding to the local radiative center of a pixel
  INTEGER(KIND=Int4)::X_Lrc_Emiss_Chn14
  INTEGER(KIND=Int4)::X_Lrc_Emiss_Chn7
  INTEGER(KIND=Int4)::X_Lrc_Refl_Chn2
  
  !-The index (in the scan line direction)
  !corresponding to the local radiative center of a pixel
  INTEGER(KIND=Int4)::Y_Lrc_Emiss_Chn14
  INTEGER(KIND=Int4)::Y_Lrc_Emiss_Chn7
  INTEGER(KIND=Int4)::Y_Lrc_Refl_Chn2
  
  !-The index (in the scan line element direction)
  !corresponding to the local radiative center of a pixel
  INTEGER(KIND=Int4)::X_Lrc_Rev_Emiss_Chn14
  INTEGER(KIND=Int4)::X_Lrc_Rev_Emiss_Chn7
  INTEGER(KIND=Int4)::X_Lrc_Rev_Refl_Chn2
  
  !-The index (in the scan line direction)
  !corresponding to the local radiative center of a pixel
  INTEGER(KIND=Int4)::Y_Lrc_Rev_Emiss_Chn14
  INTEGER(KIND=Int4)::Y_Lrc_Rev_Emiss_Chn7
  INTEGER(KIND=Int4)::Y_Lrc_Rev_Refl_Chn2
  
  !-Surface temperature
  REAL(KIND=Real4)::Tsfc
  
  REAL(KIND=Real4)::Rad_Chn14_Sfc
  REAL(KIND=Real4)::B_Chn14_Sfc
  REAL(KIND=Real4)::Rad_Emiss_Chn7_Sfc
  REAL(KIND=Real4)::BT_Chn14_Sfc
  REAL(KIND=Real4)::Rad_Chn7_Sfc
  REAL(KIND=Real4)::Emiss_Chn7_Tmpy
  
  REAL(KIND=Real4)::Refl_Chn2_LUT_Value
  
  !-Ch 11 (3.9 um) clear sky TOA radiance
  REAL(KIND=Real4)::Rad_Clr_Chn7
  
  !-Ch 14 (11 um) clear sky TOA radiance
  REAL(KIND=Real4)::Rad_Clr_Chn14
  
  !-Ch 7 (3.9 um) cloud emissivity
  REAL(KIND=Real4)::Emiss_Chn7
  REAL(KIND=Real4)::Optical_Depth_Chn7
  
  !-Ch 14 (11 um) cloud emissivity
  REAL(KIND=Real4)::Emiss_Chn14
  REAL(KIND=Real4)::Optical_Depth_Chn14
  
  REAL(KIND=Real4)::Cldt_Diff
  
  !-Emiss 3.9 um for single layered clouds
  REAL(KIND=Real4), DIMENSION(:,:), POINTER::Emiss_Chn7_Tot => NULL()
  
  !-Emiss 11 um for single layered clouds
  REAL(KIND=Real4), DIMENSION(:,:), POINTER::Emiss_Chn14_Tot => NULL()
  
  !-Emiss 11 um for single layered clouds at the LRC
  REAL(KIND=Real4), DIMENSION(:,:), POINTER::Emiss_Chn14_Tot_Lrc => NULL()
  
  REAL(KIND=Real4), DIMENSION(:,:), POINTER::Emiss_Chn7_Ratio => NULL()
  REAL(KIND=Real4), DIMENSION(:,:), POINTER::Emiss_Chn7_Atm_Corrected => NULL()
  
  !-Needed for gradient filter input
  INTEGER(KIND=Int1), DIMENSION(:,:), POINTER::BT_Chn14_Msk => NULL()
  INTEGER(KIND=Int1), DIMENSION(:,:), POINTER::BT_Chn7_Msk => NULL()
  INTEGER(KIND=Int1), DIMENSION(:,:), POINTER::Refl_Chn2_Msk => NULL()
  
  !-Needed for gradient filter output
  INTEGER(KIND=Int4), DIMENSION(:,:), POINTER::Xgrad_Emiss_Chn14 => NULL()
  INTEGER(KIND=Int4), DIMENSION(:,:), POINTER::Xgrad_Rev_Emiss_Chn14 => NULL()
  
  INTEGER(KIND=Int4), DIMENSION(:,:), POINTER::Xgrad_Emiss_Chn7 => NULL()
  INTEGER(KIND=Int4), DIMENSION(:,:), POINTER::Xgrad_Rev_Emiss_Chn7 => NULL()
  
  INTEGER(KIND=Int4), DIMENSION(:,:), POINTER::Xgrad_Refl_Chn2 => NULL()
  INTEGER(KIND=Int4), DIMENSION(:,:), POINTER::Xgrad_Rev_Refl_Chn2 => NULL()
  
  !-Needed for gradient filter output
  INTEGER(KIND=Int4), DIMENSION(:,:), POINTER::Ygrad_Emiss_Chn14 => NULL()
  INTEGER(KIND=Int4), DIMENSION(:,:), POINTER::Ygrad_Rev_Emiss_Chn14 => NULL()
  
  INTEGER(KIND=Int4), DIMENSION(:,:), POINTER::Ygrad_Emiss_Chn7 => NULL()
  INTEGER(KIND=Int4), DIMENSION(:,:), POINTER::Ygrad_Rev_Emiss_Chn7 => NULL()
  
  INTEGER(KIND=Int4), DIMENSION(:,:), POINTER::Ygrad_Refl_Chn2 => NULL()
  INTEGER(KIND=Int4), DIMENSION(:,:), POINTER::Ygrad_Rev_Refl_Chn2 => NULL()
  INTEGER(KIND=Int4), DIMENSION(:,:), POINTER::Nsteps_Grad_Rev_Refl_Chn2 => NULL()
  
  !-Ch 7 (3.9 um) clear sky radiance profile
  REAL(KIND=Real4), DIMENSION(:), POINTER::Rad_Atm_Chn7
  
  !-Ch 14 (11 um) clear sky radiance profile
  REAL(KIND=Real4), DIMENSION(:), POINTER::Rad_Atm_Chn14
  
  !-Ch 7 (3.9 um) clear sky transmittance profile
  REAL(KIND=Real4), DIMENSION(:), POINTER::Trans_Atm_Chn7
  
  !-Ch 14 (11 um) clear sky transmittance profile
  REAL(KIND=Real4), DIMENSION(:), POINTER::Trans_Atm_Chn14
  
  !-Ch 7 (3.9 um) blackbody cloud profile
  REAL(KIND=Real4), DIMENSION(:), POINTER::Cld_Prof_Chn7
  
  !-Ch 14 (11 um) blackbody cloud profile
  REAL(KIND=Real4), DIMENSION(:), POINTER::Cld_Prof_Chn14
  
  !-vertical temperature profile
  REAL(KIND=Real4), DIMENSION(:), POINTER::Tlev
  
  !-vertical pressure profile
  REAL(KIND=Real4), DIMENSION(:), POINTER::Plev
  
  !-Ch 14 (11 um) spatial stddev
  REAL(KIND=Real4), DIMENSION(:,:), ALLOCATABLE::BT_Chn14_StdDev_Tmpy
  REAL(KIND=Real4), DIMENSION(:,:), ALLOCATABLE::BT_Chn7_StdDev_Tmpy
  REAL(KIND=Real4), DIMENSION(:,:), ALLOCATABLE::Refl_Chn2_StdDev_Tmpy
  
  !-Ch 14 (11 um) spatial mean
  REAL(KIND=Real4), DIMENSION(:,:), ALLOCATABLE::BT_Chn14_Mean
  REAL(KIND=Real4), DIMENSION(:,:), ALLOCATABLE::BT_Chn7_Mean
  REAL(KIND=Real4), DIMENSION(:,:), ALLOCATABLE::Refl_Chn2_Mean
  
  !-Ch 14 (11 um) spatial minimum value
  REAL(KIND=Real4), DIMENSION(:,:), ALLOCATABLE::BT_Chn14_Min
  REAL(KIND=Real4), DIMENSION(:,:), ALLOCATABLE::BT_Chn7_Min
  REAL(KIND=Real4), DIMENSION(:,:), ALLOCATABLE::Refl_Chn2_Min
  
  !-Ch 14 (11 um) spatial maximum value
  REAL(KIND=Real4), DIMENSION(:,:), ALLOCATABLE::BT_Chn14_Max
  REAL(KIND=Real4), DIMENSION(:,:), ALLOCATABLE::BT_Chn7_Max
  REAL(KIND=Real4), DIMENSION(:,:), ALLOCATABLE::Refl_Chn2_Max
  
  REAL(KIND=Real4), DIMENSION(:,:), POINTER::BT_Chn14_StdDev => NULL()
  REAL(KIND=Real4), DIMENSION(:,:), POINTER::BT_Chn14_StdDev_Lrc => NULL()
  REAL(KIND=Real4), DIMENSION(:,:), POINTER::BT_Chn14_Cld_Edge => NULL()
  
  REAL(KIND=Real4), DIMENSION(:,:), POINTER::BT_Chn7_StdDev => NULL()
  REAL(KIND=Real4), DIMENSION(:,:), POINTER::BT_Chn7_StdDev_Lrc => NULL()
  REAL(KIND=Real4), DIMENSION(:,:), POINTER::BT_Chn7_Cld_Edge => NULL()
  
  REAL(KIND=Real4), DIMENSION(:,:), POINTER::Refl_Chn2_StdDev => NULL()
  REAL(KIND=Real4), DIMENSION(:,:), POINTER::Refl_Chn2_StdDev_Lrc => NULL()
  REAL(KIND=Real4), DIMENSION(:,:), POINTER::Refl_Chn2_Cld_Edge => NULL()
  REAL(KIND=Real4), DIMENSION(:,:), POINTER::Refl_Chn2_Cld => NULL()
  REAL(KIND=Real4), DIMENSION(:,:), POINTER::Refl_Chn2_Diff => NULL()
  
  REAL(KIND=Real4), DIMENSION(:,:), POINTER::Tsfc_Obs => NULL()
  REAL(KIND=Real4), DIMENSION(:,:), POINTER::Tsfc_Bias => NULL()
  
  REAL(KIND=Real4), DIMENSION(:,:), POINTER::Fog_Prob_Refl_Chn2_Tsfc_Bias => NULL()
  REAL(KIND=Real4), DIMENSION(:,:), POINTER::Fog_Prob_Refl_Chn2_BT_Chn14_Diff => NULL()
  REAL(KIND=Real4), DIMENSION(:,:), POINTER::Fog_Prob_Emiss_Chn14_Tsfc_Bias => NULL()
  REAL(KIND=Real4), DIMENSION(:,:), POINTER::Fog_Prob_Emiss_Chn14_BT_Chn14_Diff => NULL()
  REAL(KIND=Real4), DIMENSION(:,:), POINTER::Fog_Prob_Emiss_Chn7_Tsfc_Bias => NULL()
  REAL(KIND=Real4), DIMENSION(:,:), POINTER::Fog_Prob_Emiss_Chn7_BT_Chn14_Diff => NULL()
  REAL(KIND=Real4), DIMENSION(:,:), POINTER::Fog_Prob => NULL()
  REAL(KIND=Real4), DIMENSION(:,:), POINTER::Fog_Depth => NULL()
  
  !---------------------------------------------------------------
  ! End of local variable declaration
  !---------------------------------------------------------------

  !---------------------------------------------------------------
  ! Assign GEOCAT output structure variables to local pointers
  !---------------------------------------------------------------
  
  BT_Chn14_Msk => Out2(Ialgo)%I1_Generic1
  BT_Chn7_Msk => Out2(Ialgo)%I1_Generic2
  Refl_Chn2_Msk => Out2(Ialgo)%I1_Generic3
  
  Emiss_Chn7_Ratio => Out2(Ialgo)%Ems7_Ratio
  Emiss_Chn7_Atm_Corrected => Out2(Ialgo)%Ems7_Atmos_Corrected
  
  Xgrad_Emiss_Chn14 => Out2(Ialgo)%Xgrad_Emiss14
  Ygrad_Emiss_Chn14 => Out2(Ialgo)%Ygrad_Emiss14
  Xgrad_Rev_Emiss_Chn14 => Out2(Ialgo)%Xgrad_Rev_Emiss14
  Ygrad_Rev_Emiss_Chn14 => Out2(Ialgo)%Ygrad_Rev_Emiss14
  
  Xgrad_Emiss_Chn7 => Out2(Ialgo)%Xgrad_Emiss7
  Ygrad_Emiss_Chn7 => Out2(Ialgo)%Ygrad_Emiss7
  Xgrad_Rev_Emiss_Chn7 => Out2(Ialgo)%Xgrad_Rev_Emiss7
  Ygrad_Rev_Emiss_Chn7 => Out2(Ialgo)%Ygrad_Rev_Emiss7
  
  Xgrad_Refl_Chn2 => Out2(Ialgo)%Xgrad_Ref2
  Ygrad_Refl_Chn2 => Out2(Ialgo)%Ygrad_Ref2
  Xgrad_Rev_Refl_Chn2 => Out2(Ialgo)%Xgrad_Rev_Ref2
  Ygrad_Rev_Refl_Chn2 => Out2(Ialgo)%Ygrad_Rev_Ref2
  Nsteps_Grad_Rev_Refl_Chn2 => Out2(Ialgo)%Nsteps_Grad_Rev_Ref2
  
  Emiss_Chn7_Tot => Out2(Ialgo)%Emiss_Ch7_Tot
  Emiss_Chn14_Tot => Out2(Ialgo)%Emiss_Ch14_Tot
  Emiss_Chn14_Tot_Lrc => Out2(Ialgo)%Emiss_Ch14_Tot_Lrc
  
  BT_Chn14_StdDev => Out2(Ialgo)%BT14_StdDev
  BT_Chn14_StdDev_Lrc => Out2(Ialgo)%BT14_StdDev_Lrc
  BT_Chn14_Cld_Edge => Out2(Ialgo)%BT14_Cloud_Edge
  
  BT_Chn7_StdDev => Out2(Ialgo)%BT7_StdDev
  BT_Chn7_StdDev_Lrc => Out2(Ialgo)%BT7_StdDev_Lrc
  BT_Chn7_Cld_Edge => Out2(Ialgo)%BT7_Cloud_Edge
  
  Refl_Chn2_StdDev => Out2(Ialgo)%Ref2_StdDev
  Refl_Chn2_StdDev_Lrc => Out2(Ialgo)%Ref2_StdDev_Lrc
  Refl_Chn2_Cld_Edge => Out2(Ialgo)%Ref2_Cloud_Edge
  Refl_Chn2_Cld => Out2(Ialgo)%Ref2_Cloud
  Refl_Chn2_Diff => Out2(Ialgo)%Ref2_Diff
  
  Tsfc_Obs => Out2(Ialgo)%Tsfc
  Tsfc_Bias => Out2(Ialgo)%Tsfc_Bias
  
  Fog_Prob_Refl_Chn2_Tsfc_Bias => Out2(Ialgo)%Fog_Prob_Ref2_Tsfc_Bias
  Fog_Prob_Refl_Chn2_BT_Chn14_Diff => Out2(Ialgo)%Fog_Prob_Ref2_BT14_Diff
  Fog_Prob_Emiss_Chn14_Tsfc_Bias => Out2(Ialgo)%Fog_Prob_Emiss14_Tsfc_Bias
  Fog_Prob_Emiss_Chn14_BT_Chn14_Diff => Out2(Ialgo)%Fog_Prob_Emiss14_BT14_Diff
  Fog_Prob_Emiss_Chn7_Tsfc_Bias => Out2(Ialgo)%Fog_Prob_Ems7_Tsfc_Bias
  Fog_Prob_Emiss_Chn7_BT_Chn14_Diff => Out2(Ialgo)%Fog_Prob_Ems7_BT14_Diff
  Fog_Prob => Out2(Ialgo)%Fog_Prob
  Fog_Depth => Out2(Ialgo)%Fog_Depth
  print *,'ialgo fog',ialgo
  
  !---------------------------------------------------------------
  ! Read LUT from HDF file only once - on first scan segement
  !---------------------------------------------------------------

  IF (Sat%Iseg == 0) THEN
    
    CALL Display_Message(Routine_Name, &
                         "Reading in fog LUT, "//Trim(LUT_Filename), &
		         Sym%INFORMATION)
    
    CALL Read_Fog_LUT(Trim(Out2(Ialgo)%Ancil_Subdir),LUT_Filename)
    
    CALL Display_Message(Routine_Name, &
                         "Finished reading fog LUT", &
		         Sym%INFORMATION)  
  
    EMS7_DELTA_PROB = 0.0
    EMS7_DELTA_DEPTH = 0.0
    
    IF (Scinfo(Sc_Ind)%Series == "goes") THEN
      EMS7_DELTA_PROB = -0.2
    ENDIF
    
    IF (Scinfo(Sc_Ind)%Series == "meteosat2") THEN
      EMS7_DELTA_DEPTH = 0.2
    ENDIF
  
  ENDIF
  
  !+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  ! Begin first loop over each pixel - variable initialization
  !+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  
  !---------------------------------------------------------------
  ! Loop over each scan line
  !---------------------------------------------------------------
  print *,'iline fog',sat%ny,'elem',sat%nx
  
  Line_Loop1: DO Iline=1, Sat%Ny
    
    !---------------------------------------------------------------
    ! Loop over each element of the scan line
    !---------------------------------------------------------------
    
    Elem_Loop1: DO Ielem=1, Sat%Nx
      
      !---------------------------------------------------------------
      ! Initialize intermediate variables
      !---------------------------------------------------------------
      
      BT_Chn14_Msk(Ielem,Iline) = Sym%NO
      BT_Chn7_Msk(Ielem,Iline) = Sym%NO
      Refl_Chn2_Msk(Ielem,Iline) = Sym%NO
      
      Xgrad_Emiss_Chn14(Ielem,Iline) = Missing_Value_Int4
      Ygrad_Emiss_Chn14(Ielem,Iline) = Missing_Value_Int4
      Xgrad_Rev_Emiss_Chn14(Ielem,Iline) = Missing_Value_Int4
      Ygrad_Rev_Emiss_Chn14(Ielem,Iline) = Missing_Value_Int4
  
      Xgrad_Emiss_Chn7(Ielem,Iline) = Missing_Value_Int4
      Ygrad_Emiss_Chn7(Ielem,Iline) = Missing_Value_Int4
      Xgrad_Rev_Emiss_Chn7(Ielem,Iline) = Missing_Value_Int4
      Ygrad_Rev_Emiss_Chn7(Ielem,Iline) = Missing_Value_Int4
  
      Xgrad_Refl_Chn2(Ielem,Iline) = Missing_Value_Int4
      Ygrad_Refl_Chn2(Ielem,Iline) = Missing_Value_Int4
      Xgrad_Rev_Refl_Chn2(Ielem,Iline) = Missing_Value_Int4
      Ygrad_Rev_Refl_Chn2(Ielem,Iline) = Missing_Value_Int4
      Nsteps_Grad_Rev_Refl_Chn2(Ielem,Iline) = Missing_Value_Int4
      
      Emiss_Chn7_Tot(Ielem,Iline) = Missing_Value_Real4
      
      Emiss_Chn14_Tot(Ielem,Iline) = Missing_Value_Real4
      Emiss_Chn14_Tot_Lrc(Ielem,Iline) = Missing_Value_Real4
    
      Emiss_Chn7_Ratio(Ielem,Iline) = Missing_Value_Real4
      Emiss_Chn7_Atm_Corrected(Ielem,Iline) = Missing_Value_Real4
      
      BT_Chn14_StdDev(Ielem,Iline) = Missing_Value_Real4
      BT_Chn14_StdDev_Lrc(Ielem,Iline) = Missing_Value_Real4
      BT_Chn14_Cld_Edge(Ielem,Iline) = Missing_Value_Real4
      
      BT_Chn7_StdDev(Ielem,Iline) = Missing_Value_Real4
      BT_Chn7_StdDev_Lrc(Ielem,Iline) = Missing_Value_Real4
      BT_Chn7_Cld_Edge(Ielem,Iline) = Missing_Value_Real4
      
      Refl_Chn2_StdDev(Ielem,Iline) = Missing_Value_Real4
      Refl_Chn2_StdDev_Lrc(Ielem,Iline) = Missing_Value_Real4
      Refl_Chn2_Cld_Edge(Ielem,Iline) = Missing_Value_Real4
      Refl_Chn2_Cld(Ielem,Iline) = Missing_Value_Real4
      Refl_Chn2_Diff(Ielem,Iline) = Missing_Value_Real4
      
      Tsfc_Obs(Ielem,Iline) = Missing_Value_Real4
      Tsfc_Bias(Ielem,Iline) = Missing_Value_Real4
      
      Fog_Prob_Refl_Chn2_Tsfc_Bias(Ielem,Iline) = Missing_Value_Real4
      Fog_Prob_Refl_Chn2_BT_Chn14_Diff(Ielem,Iline) = Missing_Value_Real4
      Fog_Prob_Emiss_Chn14_Tsfc_Bias(Ielem,Iline) = Missing_Value_Real4
      Fog_Prob_Emiss_Chn14_BT_Chn14_Diff(Ielem,Iline) = Missing_Value_Real4
      Fog_Prob_Emiss_Chn7_Tsfc_Bias(Ielem,Iline) = Missing_Value_Real4
      Fog_Prob_Emiss_Chn7_BT_Chn14_Diff(Ielem,Iline) = Missing_Value_Real4
      Fog_Prob(Ielem,Iline) = Missing_Value_Real4
      Fog_Depth(Ielem,Iline) = Missing_Value_Real4
      
      !---------------------------------------------------------------
      ! If the current pixel is not geolocated or bad data is present, 
      ! CYCLE to the next pixel.
      !---------------------------------------------------------------
      
      IF (Sat%Space_Mask(Ielem,Iline) == Sym%SPACE .OR. &
          Sat%Bad_Pixel_Mask(14,Ielem,Iline) == Sym%YES) THEN
        
        CYCLE
        
      ENDIF
      
      !---------------------------------------------------------------
      ! Determine the output flags describing background information.
      !---------------------------------------------------------------
      
      !---------------------------------------------------------------
      ! Alias some variables to shorter names.
      !---------------------------------------------------------------
      
      !---------------------------------------------------------------
      ! Index variables alias list
      !---------------------------------------------------------------
        
      Ivza = Sat%Ivza(Ielem,Iline)
      Xnwp = Sat%X_Nwp(Ielem,Iline)
      Ynwp = Sat%Y_Nwp(Ielem,Iline)
      Itropo = Nwp%Dat(Xnwp,Ynwp)%Tropo_Level
      Isfc = Sat%Isfc(Ielem,Iline)
      Nprof = (Isfc-Itropo)+1 
        
      !---------------------------------------------------------------
      ! Profile variables alias list
      !---------------------------------------------------------------
      
      Tlev => Nwp%Dat(Xnwp,Ynwp)%Tlev(Itropo:Isfc)
      Plev => Nwp%Dat(Xnwp,Ynwp)%Plev(Itropo:Isfc)
      Tsfc = Nwp%Dat(Xnwp,Ynwp)%Tsfc
      
      !---------------------------------------------------------------
      ! Clear sky TOA radiance alias list
      !---------------------------------------------------------------
        
      Rad_Clr_Chn7 = Sat%Rad_Clr7(Ielem,Iline)
      Rad_Clr_Chn14 = Sat%Rad_Clr14(Ielem,Iline)
        
      !---------------------------------------------------------------
      ! Clear sky profile alias list
      !---------------------------------------------------------------
      
      Rad_Atm_Chn7 => Rtm(Xnwp,Ynwp)%D(Ivza)%Rad_Atm_Clr7(Itropo:Isfc)
      Rad_Atm_Chn14 => Rtm(Xnwp,Ynwp)%D(Ivza)%Rad_Atm_Clr14(Itropo:Isfc)
      
      !---------------------------------------------------------------
      ! Clear sky transmittance alias list
      !---------------------------------------------------------------
      
      Trans_Atm_Chn7 => Rtm(Xnwp,Ynwp)%D(Ivza)%Trans_Atm_Clr7(Itropo:Isfc)
      Trans_Atm_Chn14 => Rtm(Xnwp,Ynwp)%D(Ivza)%Trans_Atm_Clr14(Itropo:Isfc)
        
      !---------------------------------------------------------------
      ! Black cloud profile alias list
      !---------------------------------------------------------------
      
      Cld_Prof_Chn7 => Rtm(Xnwp,Ynwp)%D(Ivza)%Cloud_Prof7(Itropo:Isfc)
      Cld_Prof_Chn14 => Rtm(Xnwp,Ynwp)%D(Ivza)%Cloud_Prof14(Itropo:Isfc)
      
      !---------------------------------------------------------------
      ! Calculate sst/lst
      !---------------------------------------------------------------
      
      Rad_Chn14_Sfc = (Sat%Rad14(Ielem,Iline) - Rad_Atm_Chn14(Nprof))/Trans_Atm_Chn14(Nprof)
      B_Chn14_Sfc = Rad_Chn14_Sfc/Sat%Sfc_Emiss14(Ielem,Iline)
      Tsfc_Obs(Ielem,Iline) = Planck_Temp_Fast(14,B_Chn14_Sfc)
      
      Tsfc_Bias(Ielem,Iline) = Tsfc_Obs(Ielem,Iline) - Tsfc
      
      !---------------------------------------------------------------
      ! Calculate atmospherically corrected ems7
      !---------------------------------------------------------------
      
      Emiss_Chn7_Atm_Corrected(Ielem,Iline) = 0.0
        
      IF (Sat%Solzen(Ielem,Iline) > 90.0) THEN
	
	BT_Chn14_Sfc = Planck_Temp_Fast(14,Rad_Chn14_Sfc)
        Rad_Chn7_Sfc = MAX(0.0,(Sat%Rad7(Ielem,Iline) - Rad_Atm_Chn7(Nprof))/Trans_Atm_Chn7(Nprof))
      
        IF (BT_Chn14_Sfc > 0.0) THEN
          Rad_Emiss_Chn7_Sfc = Planck_Rad_Fast(7,BT_Chn14_Sfc)
          Emiss_Chn7_Atm_Corrected(Ielem,Iline) = Rad_Chn7_Sfc/Rad_Emiss_Chn7_Sfc
	  Emiss_Chn7_Atm_Corrected(Ielem,Iline) = MIN(5.0,MAX(0.0,Emiss_Chn7_Atm_Corrected(Ielem,Iline)))
          
        ENDIF
	
      ENDIF
      
      !---------------------------------------------------------------
      ! If the cloud type indicates ice, set the output and CYCLE
      ! to the next pixel.
      !---------------------------------------------------------------
          
      IF (Sat%Cldtype(Ielem,Iline) /= Sym%CLEAR_TYPE .AND. &
          Sat%Cldtype(Ielem,Iline) /= Sym%WATER_TYPE .AND. &
          Sat%Cldtype(Ielem,Iline) /= Sym%SUPERCOOLED_TYPE .AND. &
          Sat%Cldtype(Ielem,Iline) /= Sym%MIXED_TYPE) THEN
        
        CYCLE
  
      ENDIF
      
      !---------------------------------------------------------------
      ! Calculate cloud emissivity referenced to top of troposphere
      !---------------------------------------------------------------
      
      Emiss_Chn7 = Calculate_Cloud_Emissivity(Sat%Rad7(Ielem,Iline),Rad_Clr_Chn7,Cld_Prof_Chn7(1))
      Emiss_Chn14 = Calculate_Cloud_Emissivity(Sat%Rad14(Ielem,Iline),Rad_Clr_Chn14,Cld_Prof_Chn14(1))
      
      Emiss_Chn7 = Nadir_Cloud_Emissivity(Emiss_Chn7, Sat%Cos_Satzen(Ielem,Iline), Corrected_Tau=Optical_Depth_Chn7)
      Emiss_Chn14 = Nadir_Cloud_Emissivity(Emiss_Chn14, Sat%Cos_Satzen(Ielem,Iline), Corrected_Tau=Optical_Depth_Chn14)
      
      Emiss_Chn7_Tot(Ielem,Iline) = Emiss_Chn7
      Emiss_Chn14_Tot(Ielem,Iline) = Emiss_Chn14
      
      !---------------------------------------------------------------
      ! Make sure that the cloud emissivity has a value greater than
      ! zero whenever the cloud mask returns cloudy.
      !---------------------------------------------------------------
      
      IF (Sat%Cldmask(Ielem,Iline) == Sym%CLOUDY .AND. &
          Emiss_Chn7_Tot(Ielem,Iline) == Missing_Value_Real4) THEN
        
        Emiss_Chn7_Tot(Ielem,Iline) = 1.0e-08
      
      ENDIF
      
      IF (Sat%Cldmask(Ielem,Iline) == Sym%CLOUDY .AND. &
          Emiss_Chn14_Tot(Ielem,Iline) == Missing_Value_Real4) THEN
        
        Emiss_Chn14_Tot(Ielem,Iline) = 0.02
      
      ENDIF
      
      IF (Emiss_Chn14_Tot(Ielem,Iline) == Missing_Value_Real4) Emiss_Chn14_Tot(Ielem,Iline) = 0.0
      IF (Emiss_Chn7_Tot(Ielem,Iline) == Missing_Value_Real4) Emiss_Chn7_Tot(Ielem,Iline) = 0.0

      !---------------------------------------------------------------
      ! Set a flag so that the gradient filter will only look to
      ! identify the local radiative center of pixels that are
      ! water clouds.
      !---------------------------------------------------------------
      
      Refl_Chn2_Cld(Ielem,Iline) = 0.0
      Emiss_Chn7_Ratio(Ielem,Iline) = 0.0
      
      IF (Sat%Cldtype(Ielem,Iline) == Sym%WATER_TYPE .OR. &
          Sat%Cldtype(Ielem,Iline) == Sym%SUPERCOOLED_TYPE .OR. &
          Sat%Cldtype(Ielem,Iline) == Sym%MIXED_TYPE) THEN
        
        BT_Chn14_Msk(Ielem,Iline) = Sym%YES
	
	IF (Sat%Solzen(Ielem,Iline) < 90.0 .AND. &
	    Sat%Cldmask(Ielem,Iline) /= Sym%PROB_CLEAR .AND. &
	    Sat%Cldmask(Ielem,Iline) /= Sym%CLEAR) THEN
	  
	  Refl_Chn2_Msk(Ielem,Iline) = Sym%YES
	  Refl_Chn2_Cld(Ielem,Iline) = Sat%Ref2(Ielem,Iline)
	  
	  !Note that this is a temporary hack designed to prevent overshoot of cloud edge over land.
	  !Ideally a more elegant supplement to the cloud mask should be used here.
	  IF (Sat%Land_Mask(Ielem,Iline) == Sym%LAND .AND. &
	      Emiss_Chn14_Tot(Ielem,Iline) < 0.02 .AND. &
	      Sat%Ref2(Ielem,Iline) < 40.0) THEN
	    Refl_Chn2_Msk(Ielem,Iline) = Sym%NO
	    Refl_Chn2_Cld(Ielem,Iline) = 0.0
	  ENDIF
	  
	ENDIF
	
	IF (Sat%Solzen(Ielem,Iline) > 90.0 .AND. &
	    Sat%Cldmask(Ielem,Iline) /= Sym%PROB_CLEAR .AND. &
	    Sat%Cldmask(Ielem,Iline) /= Sym%CLEAR) THEN
	  
	  BT_Chn7_Msk(Ielem,Iline) = Sym%YES
	  
	ENDIF
      
      ENDIF
      
    END DO Elem_Loop1
    
    !---------------------------------------------------------------
    ! End of loop over each element of the scan line
    !---------------------------------------------------------------
    
  END DO Line_Loop1
  
  !---------------------------------------------------------------
  ! End of loop over each scan line
  !---------------------------------------------------------------
  
  !+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  ! End of first loop
  !+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  
          
  !---------------------------------------------------------------
  ! Apply a 3x3 median filter to some of the emissivity arrays and
  ! beta arrays.
  !---------------------------------------------------------------
 
  CALL Median_Filter(Refl_Chn2_Cld, 3, Sat%Space_Mask)
  CALL Median_Filter(Emiss_Chn7_Tot, 3, Sat%Space_Mask)
  CALL Median_Filter(Emiss_Chn14_Tot, 3, Sat%Space_Mask)
  CALL Median_Filter(Emiss_Chn7_Atm_Corrected, 3, Sat%Space_Mask)
  
  !---------------------------------------------------------------
  ! Find the indices of the local radiative center of each pixel.
  !---------------------------------------------------------------
  
  PRINT*,'getting gradients'
  
  CALL Gradient2D(Emiss_Chn14_Tot, &
                  Sat%Nx, &
                  Sat%Ny, &
                  BT_Chn14_Msk, &
                  0.0, &
                  1.0, &
                  0.7, &
                  Xgrad_Emiss_Chn14, &
                  Ygrad_Emiss_Chn14)
		  
  CALL Gradient2D(Emiss_Chn7_Tot, &
                  Sat%Nx, &
                  Sat%Ny, &
                  BT_Chn7_Msk, &
                  0.1, &
                  1.0, &
                  0.9, &
                  Xgrad_Emiss_Chn7, &
                  Ygrad_Emiss_Chn7)
		  
  CALL Gradient2D(Refl_Chn2_Cld, &
                  Sat%Nx, &
                  Sat%Ny, &
                  Refl_Chn2_Msk, &
                  0.0, &
                  110.0, &
                  90.0, &
                  Xgrad_Refl_Chn2, &
                  Ygrad_Refl_Chn2)
		  
  !---------------------------------------------------------------
  ! Find the indices of the reverse local radiative center of each pixel.
  !---------------------------------------------------------------
   
   PRINT*,'getting reverse gradients'
   
   CALL Find_Nearest_Edge(Emiss_Chn14_Tot, &
                          Sat%Nx, &
                          Sat%Ny, &
                          BT_Chn14_Msk, &
                          0.01, &
                          1.0, &
                          0.03, &
                          Xgrad_Rev_Emiss_Chn14, &
                          Ygrad_Rev_Emiss_Chn14, &
			  Nsteps_Out=Nsteps_Grad_Rev_Refl_Chn2, &
			  Max_Steps=50)
			   
  CALL Find_Nearest_Edge(Emiss_Chn7_Atm_Corrected, &
                         Sat%Nx, &
                         Sat%Ny, &
                         BT_Chn7_Msk, &
                         0.1, &
                         1.2, &
                         0.1, &
                         Xgrad_Rev_Emiss_Chn7, &
                         Ygrad_Rev_Emiss_Chn7, &
			 Max_Steps=50)
			   
  CALL Find_Nearest_Edge(Refl_Chn2_Cld, &
                         Sat%Nx, &
                         Sat%Ny, &
                         Refl_Chn2_Msk, &
                         0.1, &
                         110.0, &
                         0.1, &
                         Xgrad_Rev_Refl_Chn2, &
                         Ygrad_Rev_Refl_Chn2, &
			 Max_Steps=50)
			   
  PRINT*,'getting spatial uniformity'
  
  !---------------------------------------------------------------
  ! Compute some 11 micron bt statistics for 3 x 3 pixel area.
  !---------------------------------------------------------------
  
  CALL Compute_Spatial_Uniformity(1, &
                                  1, &
                                  Sat%Space_Mask, &
                                  Sat%BT14, &
                                  BT_Chn14_Mean, &
                                  BT_Chn14_Max, &
                                  BT_Chn14_Min, &
                                  BT_Chn14_StdDev_Tmpy)
				  
  !---------------------------------------------------------------
  ! Compute some 3.9 micron bt statistics for 3 x 3 pixel area.
  !---------------------------------------------------------------
  
  CALL Compute_Spatial_Uniformity(1, &
                                  1, &
                                  Sat%Space_Mask, &
                                  Sat%BT7, &
                                  BT_Chn7_Mean, &
                                  BT_Chn7_Max, &
                                  BT_Chn7_Min, &
                                  BT_Chn7_StdDev_Tmpy)
				  
  !---------------------------------------------------------------
  ! Compute some 0.65 micron ref statistics for 3 x 3 pixel area.
  !---------------------------------------------------------------
  
  CALL Compute_Spatial_Uniformity(1, &
                                  1, &
                                  Sat%Space_Mask, &
                                  Sat%Ref2, &
                                  Refl_Chn2_Mean, &
                                  Refl_Chn2_Max, &
                                  Refl_Chn2_Min, &
                                  Refl_Chn2_StdDev_Tmpy)
  
  !+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  ! Begin second loop over each pixel - create LRC arrays
  !+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  
  !---------------------------------------------------------------
  ! Loop over each scan line
  !---------------------------------------------------------------
  
  Line_Loop2: DO Iline = 1, Sat%Ny
  
    !---------------------------------------------------------------
    ! Loop over each element of the scan line
    !---------------------------------------------------------------
  
    Elem_Loop2: DO Ielem = 1, Sat%Nx
      
      Out2(Ialgo)%BT14_StdDev(Ielem,Iline) = BT_Chn14_StdDev_Tmpy(Ielem,Iline)
      Out2(Ialgo)%BT7_StdDev(Ielem,Iline) = BT_Chn7_StdDev_Tmpy(Ielem,Iline)
      Out2(Ialgo)%Ref2_StdDev(Ielem,Iline) = Refl_Chn2_StdDev_Tmpy(Ielem,Iline)
      
      !---------------------------------------------------------------
      ! If looking at space or bad data, CYCLE.
      !---------------------------------------------------------------
      
      IF (Sat%Space_Mask(Ielem,Iline) == Sym%SPACE .OR. &
          Sat%Bad_Pixel_Mask(14,Ielem,Iline) == Sym%YES) THEN
        
        CYCLE
  
      ENDIF
      
      !---------------------------------------------------------------
      ! Alias the x and y array indices of the local radiative
      ! center (lrc) and reverse lrc of each pixel to a shorter variable.
      !---------------------------------------------------------------
      
      X_Lrc_Emiss_Chn14 = Xgrad_Emiss_Chn14(Ielem,Iline)
      Y_Lrc_Emiss_Chn14 = Ygrad_Emiss_Chn14(Ielem,Iline)
      
      X_Lrc_Rev_Emiss_Chn14 = Xgrad_Rev_Emiss_Chn14(Ielem,Iline)
      Y_Lrc_Rev_Emiss_Chn14 = Ygrad_Rev_Emiss_Chn14(Ielem,Iline)
      
      X_Lrc_Emiss_Chn7 = Xgrad_Emiss_Chn7(Ielem,Iline)
      Y_Lrc_Emiss_Chn7 = Ygrad_Emiss_Chn7(Ielem,Iline)
      
      X_Lrc_Rev_Emiss_Chn7 = Xgrad_Rev_Emiss_Chn7(Ielem,Iline)
      Y_Lrc_Rev_Emiss_Chn7 = Ygrad_Rev_Emiss_Chn7(Ielem,Iline)
      
      X_Lrc_Refl_Chn2 = Xgrad_Refl_Chn2(Ielem,Iline)
      Y_Lrc_Refl_Chn2 = Ygrad_Refl_Chn2(Ielem,Iline)
      
      X_Lrc_Rev_Refl_Chn2 = Xgrad_Rev_Refl_Chn2(Ielem,Iline)
      Y_Lrc_Rev_Refl_Chn2 = Ygrad_Rev_Refl_Chn2(Ielem,Iline)
      
      !---------------------------------------------------------------
      ! Create arrays of the LRC variables
      !---------------------------------------------------------------
      
      IF (X_Lrc_Emiss_Chn14 > 0 .AND. Y_Lrc_Emiss_Chn14 > 0) THEN
        
	Emiss_Chn14_Tot_Lrc(Ielem,Iline) = Emiss_Chn14_Tot(X_Lrc_Emiss_Chn14,Y_Lrc_Emiss_Chn14)
	BT_Chn14_StdDev_Lrc(Ielem,Iline) = BT_Chn14_StdDev_Tmpy(X_Lrc_Emiss_Chn14,Y_Lrc_Emiss_Chn14)
       
      ENDIF
      
      !---------------------------------------------------------------
      ! Create arrays of the LRC variables
      !---------------------------------------------------------------
      
      IF (X_Lrc_Emiss_Chn7 > 0 .AND. Y_Lrc_Emiss_Chn7 > 0) THEN
        
	BT_Chn7_StdDev_Lrc(Ielem,Iline) = BT_Chn7_StdDev_Tmpy(X_Lrc_Emiss_Chn7,Y_Lrc_Emiss_Chn7)
       
      ENDIF
      
      !---------------------------------------------------------------
      ! Create arrays of the LRC variables
      !---------------------------------------------------------------
      
      IF (X_Lrc_Refl_Chn2 > 0 .AND. Y_Lrc_Refl_Chn2 > 0) THEN
        
	Refl_Chn2_StdDev_Lrc(Ielem,Iline) = Refl_Chn2_StdDev_Tmpy(X_Lrc_Refl_Chn2,Y_Lrc_Refl_Chn2)
       
      ENDIF
      
      !---------------------------------------------------------------
      ! Create arrays of the reverse LRC variables
      !---------------------------------------------------------------
      
      IF (X_Lrc_Rev_Emiss_Chn14 > 0 .AND. Y_Lrc_Rev_Emiss_Chn14 > 0) THEN
        
	BT_Chn14_Cld_Edge(Ielem,Iline) = Sat%BT14(X_Lrc_Rev_Emiss_Chn14,Y_Lrc_Rev_Emiss_Chn14)
       
      ENDIF
      
      IF (X_Lrc_Rev_Emiss_Chn7 > 0 .AND. Y_Lrc_Rev_Emiss_Chn7 > 0) THEN
        
	BT_Chn7_Cld_Edge(Ielem,Iline) = Sat%BT14(X_Lrc_Rev_Emiss_Chn7,Y_Lrc_Rev_Emiss_Chn7)
       
      ENDIF
      
      IF (X_Lrc_Rev_Refl_Chn2 > 0 .AND. Y_Lrc_Rev_Refl_Chn2 > 0) THEN
        
	Refl_Chn2_Cld_Edge(Ielem,Iline) = Sat%BT14(X_Lrc_Rev_Refl_Chn2,Y_Lrc_Rev_Refl_Chn2)
	Refl_Chn2_Diff(Ielem,Iline) = Sat%Ref2(Ielem,Iline) - Sat%Ref2(X_Lrc_Rev_Refl_Chn2,Y_Lrc_Rev_Refl_Chn2)
       
      ENDIF
      
    END DO Elem_Loop2
    
    !---------------------------------------------------------------
    ! End of loop over each element of the scan line
    !---------------------------------------------------------------
    
  END DO Line_Loop2
  
  !---------------------------------------------------------------
  ! End of loop over each scan line
  !---------------------------------------------------------------
  
  !+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  ! End of second loop
  !+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  
  !---------------------------------------------------------------
  ! Apply a 3x3 median filter to the LRC and reverse LRC variables
  ! to remove most artifacts.
  !---------------------------------------------------------------
 
  CALL Median_Filter(Emiss_Chn14_Tot_Lrc, 3, Sat%Space_Mask)
  CALL Median_Filter(BT_Chn14_Cld_Edge, 3, Sat%Space_Mask)
  CALL Median_Filter(BT_Chn7_Cld_Edge, 3, Sat%Space_Mask)
  CALL Median_Filter(Refl_Chn2_Cld_Edge, 3, Sat%Space_Mask)
  CALL Median_Filter(Refl_Chn2_Diff, 3, Sat%Space_Mask)
  CALL Median_Filter(BT_Chn14_StdDev_Lrc, 3, Sat%Space_Mask)
 
  !+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  ! Begin third loop over each pixel - determine first guess
  ! fog mask
  !+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  
  !---------------------------------------------------------------
  ! Loop over each scan line
  !---------------------------------------------------------------
  
  Line_Loop3: DO Iline = 1, Sat%Ny
  
    !---------------------------------------------------------------
    ! Loop over each element of the scan line
    !---------------------------------------------------------------
  
    Elem_Loop3: DO Ielem = 1, Sat%Nx
      
      !---------------------------------------------------------------
      ! If looking at space or bad data, CYCLE.
      !---------------------------------------------------------------
      
      IF (Sat%Space_Mask(Ielem,Iline) == Sym%SPACE .OR. &
          Sat%Bad_Pixel_Mask(14,Ielem,Iline) == Sym%YES) THEN
        
        CYCLE
  
      ENDIF
      
      !---------------------------------------------------------------
      ! Alias the x and y array indices of the local radiative
      ! center (lrc) and reverse lrc of each pixel to a shorter variable.
      !---------------------------------------------------------------
      
      X_Lrc_Emiss_Chn14 = Xgrad_Emiss_Chn14(Ielem,Iline)
      Y_Lrc_Emiss_Chn14 = Ygrad_Emiss_Chn14(Ielem,Iline)
      
      X_Lrc_Rev_Emiss_Chn14 = Xgrad_Rev_Emiss_Chn14(Ielem,Iline)
      Y_Lrc_Rev_Emiss_Chn14 = Ygrad_Rev_Emiss_Chn14(Ielem,Iline)
      
      !---------------------------------------------------------------
      ! If the cloud type indicates ice, CYCLE.
      !---------------------------------------------------------------
      
      IF (Sat%Cldtype(Ielem,Iline) /= Sym%WATER_TYPE .AND. &
          Sat%Cldtype(Ielem,Iline) /= Sym%SUPERCOOLED_TYPE .AND. &
          Sat%Cldtype(Ielem,Iline) /= Sym%MIXED_TYPE) THEN
        
        CYCLE
  
      ENDIF
      
      !---------------------------------------------------------------
      ! Begin the fog detection tests.
      !---------------------------------------------------------------
      
      !---------------------------------------------------------------
      ! Calculate the BT difference between the current pixel and
      ! cloud edge.
      !---------------------------------------------------------------
      
      Cldt_Diff = 999
      
      IF (BT_Chn14_Cld_Edge(Ielem,Iline) /= Missing_Value_Real4) THEN
      
        Cldt_Diff = ABS(Sat%BT14(Ielem,Iline) - BT_Chn14_Cld_Edge(Ielem,Iline))
	
      ENDIF
      
      Fog_Prob_Refl_Chn2_Tsfc_Bias(Ielem,Iline) = 0.0
      Fog_Prob_Refl_Chn2_BT_Chn14_Diff(Ielem,Iline) = 0.0
      Fog_Prob_Emiss_Chn14_Tsfc_Bias(Ielem,Iline) = 0.0
      Fog_Prob_Emiss_Chn14_BT_Chn14_Diff(Ielem,Iline) = 0.0
      Fog_Prob_Emiss_Chn7_Tsfc_Bias(Ielem,Iline) = 0.0
      Fog_Prob_Emiss_Chn7_BT_Chn14_Diff(Ielem,Iline) = 0.0
      Fog_Prob(Ielem,Iline) = 0.0
      
      !ref2_lut_value = ref2_diff(ielem,iline)
      !if (nsteps_grad_rev_ref2(ielem,iline) >= 50 .or. ref2_lut_value < 0.0) then
      !  ref2_lut_value = max(0.0,sat%ref2(ielem,iline) - 10.0)
      !endif
      
      !---------------------------------------------------------------
      ! Get daytime fog probability over water
      !---------------------------------------------------------------
      
      IF (Sat%Solzen(Ielem,Iline) < 80.0 .AND. &
          Sat%Sfc_Type(Ielem,Iline) == Sym%WATER_SFC) THEN
	
        Refl_Chn2_LUT_Value = Sat%Ref2(Ielem,Iline) - 2.0
	
	Refl_Chn2_LUT_Idx = Get_Irregular_LUT_Index(Refl_Chn2_LUT_Value, &
	                                         Refl_Chn2_Bin_Start_LUT, &
						 Refl_Chn2_Nbins_LUT)
						 
        Tsfc_Bias_LUT_Idx = Get_Regular_LUT_Index(ABS(Tsfc_Bias(Ielem,Iline)), &
	                                            Tsfc_Bias_Min_LUT, &
						    Tsfc_Bias_Increment_LUT, &
						    Tsfc_Bias_Nbins_LUT)
						    
        Refl_Chn2_StdDev_LUT_Idx = Get_Regular_LUT_Index(Refl_Chn2_StdDev(Ielem,Iline), &
	                                              Refl_Chn2_StdDev_Min_LUT, &
						      Refl_Chn2_StdDev_Increment_LUT, &
						      Refl_Chn2_StdDev_Nbins_LUT)
						      
        Fog_Prob_Refl_Chn2_Tsfc_Bias(Ielem,Iline) = Fog_Probability_Refl_Chn2_Tsfc_Bias_Water(Refl_Chn2_LUT_Idx, &
	                                                                            Tsfc_Bias_LUT_Idx, &
										    Refl_Chn2_StdDev_LUT_Idx)
      
      ENDIF
      
      !---------------------------------------------------------------
      ! Get daytime fog probability over land
      !---------------------------------------------------------------
      
      IF (Sat%Solzen(Ielem,Iline) < 80.0 .AND. &
          Sat%Sfc_Type(Ielem,Iline) /= Sym%WATER_SFC) THEN
	
        Refl_Chn2_LUT_Value = Sat%Ref2(Ielem,Iline) - 10.0
	
	Refl_Chn2_LUT_Idx = Get_Irregular_LUT_Index(Refl_Chn2_LUT_Value, &
	                                         Refl_Chn2_Bin_Start_LUT, &
						 Refl_Chn2_Nbins_LUT)
						 
        Tsfc_Bias_LUT_Idx = Get_Regular_LUT_Index(ABS(Tsfc_Bias(Ielem,Iline)), &
	                                            Tsfc_Bias_Min_LUT, &
						    Tsfc_Bias_Increment_LUT, &
						    Tsfc_Bias_Nbins_LUT)
						    
        Refl_Chn2_StdDev_LUT_Idx = Get_Regular_LUT_Index(Refl_Chn2_StdDev(Ielem,Iline), &
	                                              Refl_Chn2_StdDev_Min_LUT, &
						      Refl_Chn2_StdDev_Increment_LUT, &
						      Refl_Chn2_StdDev_Nbins_LUT)
						      
        Fog_Prob_Refl_Chn2_Tsfc_Bias(Ielem,Iline) = Fog_Probability_Refl_Chn2_Tsfc_Bias_Land(Refl_Chn2_LUT_Idx, &
	                                                                           Tsfc_Bias_LUT_Idx, &
										   Refl_Chn2_StdDev_LUT_Idx)
      
      ENDIF
      
      !---------------------------------------------------------------
      ! Get nighttime fog probability over water
      !---------------------------------------------------------------
      
      Emiss_Chn7_Tmpy = Emiss_Chn7_Atm_Corrected(Ielem,Iline) + EMS7_DELTA_PROB
      
      IF (Sat%Solzen(Ielem,Iline) > 90.0 .AND. &
          Sat%Sfc_Type(Ielem,Iline) == Sym%WATER_SFC .AND. &
	  Emiss_Chn7_Tmpy > Emiss_Chn7_Bin_Start_LUT(1)) THEN
	
        Emiss_Chn7_LUT_Idx = Get_Irregular_LUT_Index(Emiss_Chn7_Tmpy, &
	                                         Emiss_Chn7_Bin_Start_LUT, &
						 Emiss_Chn7_Nbins_LUT)
						 
        Tsfc_Bias_LUT_Idx = Get_Regular_LUT_Index(ABS(Tsfc_Bias(Ielem,Iline)), &
	                                            Tsfc_Bias_Min_LUT, &
						    Tsfc_Bias_Increment_LUT, &
						    Tsfc_Bias_Nbins_LUT)
						    
        BT_Chn14_StdDev_LUT_Idx = Get_Regular_LUT_Index(BT_Chn14_StdDev(Ielem,Iline), &
	                                              BT_Chn14_StdDev_Min_LUT, &
						      BT_Chn14_StdDev_Increment_LUT, &
						      BT_Chn14_StdDev_Nbins_LUT)
						      
        Fog_Prob_Emiss_Chn7_Tsfc_Bias(Ielem,Iline) = Fog_Probability_Emiss_Chn7_Tsfc_Bias_Water(Emiss_Chn7_LUT_Idx, &
	                                                                            Tsfc_Bias_LUT_Idx, &
										    BT_Chn14_StdDev_LUT_Idx)
      
      ENDIF
      
      !---------------------------------------------------------------
      ! Get nighttime fog probability over land
      !---------------------------------------------------------------
      
      IF (Sat%Solzen(Ielem,Iline) > 90.0 .AND. &
          Sat%Sfc_Type(Ielem,Iline) /= Sym%WATER_SFC .AND. &
	  Emiss_Chn7_Tmpy > Emiss_Chn7_Bin_Start_LUT(1)) THEN
	
	Emiss_Chn7_LUT_Idx = Get_Irregular_LUT_Index(Emiss_Chn7_Tmpy, &
	                                         Emiss_Chn7_Bin_Start_LUT, &
						 Emiss_Chn7_Nbins_LUT)
						 
        Tsfc_Bias_LUT_Idx = Get_Regular_LUT_Index(ABS(Tsfc_Bias(Ielem,Iline)), &
	                                            Tsfc_Bias_Min_LUT, &
						    Tsfc_Bias_Increment_LUT, &
						    Tsfc_Bias_Nbins_LUT)
						    
        BT_Chn14_StdDev_LUT_Idx = Get_Regular_LUT_Index(BT_Chn14_StdDev(Ielem,Iline), &
	                                              BT_Chn14_StdDev_Min_LUT, &
						      BT_Chn14_StdDev_Increment_LUT, &
						      BT_Chn14_StdDev_Nbins_LUT)
						      
        Fog_Prob_Emiss_Chn7_Tsfc_Bias(Ielem,Iline) = Fog_Probability_Emiss_Chn7_Tsfc_Bias_Land(Emiss_Chn7_LUT_Idx, &
	                                                                           Tsfc_Bias_LUT_Idx, &
										   BT_Chn14_StdDev_LUT_Idx)
      
      ENDIF
      
      !---------------------------------------------------------------
      ! Get anytime fog probability over water
      !---------------------------------------------------------------
      
      IF (Sat%Sfc_Type(Ielem,Iline) == Sym%WATER_SFC .AND. &
          Emiss_Chn14_Tot(Ielem,Iline) >= Emiss_Chn14_Bin_Start_LUT(1)) THEN
	
        Emiss_Chn14_LUT_Idx = Get_Irregular_LUT_Index(Emiss_Chn14_Tot(Ielem,Iline), &
	                                            Emiss_Chn14_Bin_Start_LUT, &
						    Emiss_Chn14_Nbins_LUT)
						 
        Tsfc_Bias_LUT_Idx = Get_Regular_LUT_Index(ABS(Tsfc_Bias(Ielem,Iline)), &
	                                            Tsfc_Bias_Min_LUT, &
						    Tsfc_Bias_Increment_LUT, &
						    Tsfc_Bias_Nbins_LUT)
						    
        BT_Chn14_StdDev_LUT_Idx = Get_Regular_LUT_Index(BT_Chn14_StdDev(Ielem,Iline), &
	                                              BT_Chn14_StdDev_Min_LUT, &
						      BT_Chn14_StdDev_Increment_LUT, &
						      BT_Chn14_StdDev_Nbins_LUT)
						      
        Fog_Prob_Emiss_Chn14_Tsfc_Bias(Ielem,Iline) = Fog_Probability_Emiss_Chn14_Tsfc_Bias_Water(Emiss_Chn14_LUT_Idx, &
	                                                                                  Tsfc_Bias_LUT_Idx, &
										          BT_Chn14_StdDev_LUT_Idx)
      
      ENDIF
      
      !---------------------------------------------------------------
      ! Get anytime fog probability over land
      !---------------------------------------------------------------
      
      IF (Sat%Sfc_Type(Ielem,Iline) /= Sym%WATER_SFC .AND. &
          Emiss_Chn14_Tot(Ielem,Iline) >= Emiss_Chn14_Bin_Start_LUT(1)) THEN
	
        Emiss_Chn14_LUT_Idx = Get_Irregular_LUT_Index(Emiss_Chn14_Tot(Ielem,Iline), &
	                                            Emiss_Chn14_Bin_Start_LUT, &
						    Emiss_Chn14_Nbins_LUT)
						 
        Tsfc_Bias_LUT_Idx = Get_Regular_LUT_Index(ABS(Tsfc_Bias(Ielem,Iline)), &
	                                            Tsfc_Bias_Min_LUT, &
						    Tsfc_Bias_Increment_LUT, &
						    Tsfc_Bias_Nbins_LUT)
						    
        BT_Chn14_StdDev_LUT_Idx = Get_Regular_LUT_Index(BT_Chn14_StdDev(Ielem,Iline), &
	                                              BT_Chn14_StdDev_Min_LUT, &
						      BT_Chn14_StdDev_Increment_LUT, &
						      BT_Chn14_StdDev_Nbins_LUT)
						      
        Fog_Prob_Emiss_Chn14_Tsfc_Bias(Ielem,Iline) = Fog_Probability_Emiss_Chn14_Tsfc_Bias_Land(Emiss_Chn14_LUT_Idx, &
	                                                                                 Tsfc_Bias_LUT_Idx, &
										         BT_Chn14_StdDev_LUT_Idx)
      
      ENDIF
      
      IF (Sat%Solzen(Ielem,Iline) > 90.0) THEN
        
	IF (Fog_Prob_Emiss_Chn14_Tsfc_Bias(Ielem,Iline) > 0.75 .AND. Fog_Prob_Emiss_Chn7_Tsfc_Bias(Ielem,Iline) < 0.25) THEN
          Fog_Prob_Emiss_Chn7_Tsfc_Bias(Ielem,Iline) = Fog_Prob_Emiss_Chn14_Tsfc_Bias(Ielem,Iline)
        ENDIF
               
        IF (Fog_Prob_Emiss_Chn7_Tsfc_Bias(Ielem,Iline) > 0.75) THEN
	  Fog_Prob(Ielem,Iline) = Fog_Prob_Emiss_Chn7_Tsfc_Bias(Ielem,Iline)
	ENDIF
	
      ENDIF
      
      IF (Sat%Solzen(Ielem,Iline) < 80.0) THEN
        
        IF (Fog_Prob_Refl_Chn2_Tsfc_Bias(Ielem,Iline) > 0.75) THEN
	  Fog_Prob(Ielem,Iline) = Fog_Prob_Refl_Chn2_Tsfc_Bias(Ielem,Iline)
	ENDIF
      
      ENDIF
      
      IF (Sat%Solzen(Ielem,Iline) >= 80.0 .AND. Sat%Solzen(Ielem,Iline) <= 90.0) THEN
      
        IF (Fog_Prob_Emiss_Chn14_Tsfc_Bias(Ielem,Iline) > 0.75) THEN
	  Fog_Prob(Ielem,Iline) = Fog_Prob_Emiss_Chn14_Tsfc_Bias(Ielem,Iline)
	ENDIF
	
      ENDIF
      
    END DO Elem_Loop3
    
    !---------------------------------------------------------------
    ! End of loop over each element of the scan line
    !---------------------------------------------------------------
    
  END DO Line_Loop3
  
  !---------------------------------------------------------------
  ! End of loop over each scan line
  !---------------------------------------------------------------
  
  !+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  ! End of third loop
  !+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  
  CALL Median_Filter(Fog_Prob_Refl_Chn2_Tsfc_Bias, 3, Sat%Space_Mask)
  CALL Median_Filter(Fog_Prob_Emiss_Chn7_Tsfc_Bias, 3, Sat%Space_Mask)
  
  !+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  ! Begin third loop over each pixel - determine first guess
  ! fog mask
  !+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  
  !---------------------------------------------------------------
  ! Loop over each scan line
  !---------------------------------------------------------------
  
  Line_Loop4: DO Iline = 1, Sat%Ny
  
    !---------------------------------------------------------------
    ! Loop over each element of the scan line
    !---------------------------------------------------------------
  
    Elem_Loop4: DO Ielem = 1, Sat%Nx
      
      !---------------------------------------------------------------
      ! If looking at space or bad data, CYCLE.
      !---------------------------------------------------------------
      
      IF (Sat%Space_Mask(Ielem,Iline) == Sym%SPACE .OR. &
          Sat%Bad_Pixel_Mask(14,Ielem,Iline) == Sym%YES) THEN
        
        CYCLE
  
      ENDIF
      
      X_Lrc_Refl_Chn2 = Xgrad_Refl_Chn2(Ielem,Iline)
      Y_Lrc_Refl_Chn2 = Ygrad_Refl_Chn2(Ielem,Iline)
      
      X_Lrc_Emiss_Chn7 = Xgrad_Emiss_Chn7(Ielem,Iline)
      Y_Lrc_Emiss_Chn7 = Ygrad_Emiss_Chn7(Ielem,Iline)
      
      X_Lrc_Emiss_Chn14 = Xgrad_Emiss_Chn14(Ielem,Iline)
      Y_Lrc_Emiss_Chn14 = Ygrad_Emiss_Chn14(Ielem,Iline)
      
      
      IF (Sat%Solzen(Ielem,Iline) < 80.0 .AND. &
          Fog_Prob(Ielem,Iline) > 0.0 .AND. &
	  Fog_Prob(Ielem,Iline) <= 1.0) THEN
        
	IF (X_Lrc_Refl_Chn2 <= 0 .OR. Y_Lrc_Refl_Chn2 <= 0) THEN
	  Fog_Prob(Ielem,Iline) = 0.0
	ELSE IF (Fog_Prob(X_Lrc_Refl_Chn2,Y_Lrc_Refl_Chn2) <= 0.50 .AND. Sat%Ref2(Ielem,Iline) < 50.0) THEN
	  Fog_Prob(Ielem,Iline) = 0.0
	ENDIF
	
      ENDIF
      
      IF (Sat%Solzen(Ielem,Iline) > 90.0 .AND. &
          Fog_Prob(Ielem,Iline) > 0.0 .AND. &
	  Fog_Prob(Ielem,Iline) <= 1.0 .AND. &
	  X_Lrc_Emiss_Chn7 > 0 .AND. &
	  Y_Lrc_Emiss_Chn7 > 0) THEN
        
	IF (Fog_Prob(X_Lrc_Emiss_Chn7,Y_Lrc_Emiss_Chn7) <= 0.50) THEN
	  Fog_Prob(Ielem,Iline) = 0.0
	ENDIF
	
      ENDIF
      
      IF (Sat%Solzen(Ielem,Iline) > 90.0 .AND. &
          Fog_Prob(Ielem,Iline) > 0.0 .AND. &
	  Fog_Prob(Ielem,Iline) <= 1.0 .AND. &
	  ABS(Sat%Lat(Ielem,Iline)) <= 25.0 .AND. &
	  Tsfc_Bias(Ielem,Iline) > 0.5 .AND. &
	  Sat%Sfc_Type(Ielem,Iline) /= Sym%WATER_SFC) THEN
	  
        Fog_Prob(Ielem,Iline) = 0.0
	
      ENDIF
      
      IF (Sat%Solzen(Ielem,Iline) >= 80.0 .AND. &
          Sat%Solzen(Ielem,Iline) <= 90.0 .AND. &
          Fog_Prob(Ielem,Iline) > 0.0 .AND. &
	  Fog_Prob(Ielem,Iline) <= 1.0 .AND. &
	  X_Lrc_Emiss_Chn14 > 0 .AND. &
	  Y_Lrc_Emiss_Chn14 > 0) THEN
        
	IF (Fog_Prob(X_Lrc_Emiss_Chn14,Y_Lrc_Emiss_Chn14) <= 0.50) THEN
	  Fog_Prob(Ielem,Iline) = 0.0
	ENDIF
	
      ENDIF
      
      !This is temporary
      IF (Sat%Sfc_Type(Ielem,Iline) == Sym%BARE_SFC) THEN
        Fog_Prob(Ielem,Iline) = 0.0
      ENDIF
      
      IF (Sat%Solzen(Ielem,Iline) < 80.0 .AND. &
          Fog_Prob_Refl_Chn2_Tsfc_Bias(Ielem,Iline) > 0.0 .AND. &
	  Fog_Prob_Refl_Chn2_Tsfc_Bias(Ielem,Iline) <= 1.0 .AND. &
          Sat%Cldlwp(Ielem,Iline) > 0.0 .AND. &
          Sat%Cldlwp(Ielem,Iline) < 100.0) THEN
	
        Fog_Depth(Ielem,Iline) = Sat%Cldlwp(Ielem,Iline)/0.06
        
      ENDIF
      
      Emiss_Chn7_Tmpy = Emiss_Chn7_Atm_Corrected(Ielem,Iline) + EMS7_DELTA_DEPTH
      
      IF (Sat%Solzen(Ielem,Iline) > 90.0 .AND. &
          Fog_Prob_Emiss_Chn7_Tsfc_Bias(Ielem,Iline) > 0.0 .AND. &
	  Fog_Prob_Emiss_Chn7_Tsfc_Bias(Ielem,Iline) <= 1.0 .AND. &
          Emiss_Chn7_Tmpy > 0.6) THEN
          
        Fog_Depth(Ielem,Iline) = -1159.93*Emiss_Chn7_Tmpy + 1295.70
	
      ENDIF
!      if(fog_prob(ielem,iline).ne.-999.and.fog_prob(ielem,iline).ne.0.0)print *,'fog_prob',ielem,iline,fog_prob(ielem,iline)
      
    END DO Elem_Loop4
    
    !---------------------------------------------------------------
    ! End of loop over each element of the scan line
    !---------------------------------------------------------------
    
  END DO Line_Loop4
  
  !---------------------------------------------------------------
  ! End of loop over each scan line
  !---------------------------------------------------------------
  
  !+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  ! End of third loop
  !+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  
  !---------------------------------------------------------------
  ! Apply a 3x3 median filter to the fog mask to eliminate noise
  ! and spatial artifacts.
  !---------------------------------------------------------------
  
  CALL Median_Filter(Fog_Prob, 3, Sat%Space_Mask)
 
  !---------------------------------------------------------------
  ! Destroy spatial uniformity memory.
  !---------------------------------------------------------------
  
  CALL Destroy_Spatial_Uniformity(BT_Chn14_Mean, BT_Chn14_Max, &
                                  BT_Chn14_Min, BT_Chn14_StdDev_Tmpy)
				  
  CALL Destroy_Spatial_Uniformity(BT_Chn7_Mean, BT_Chn7_Max, &
                                  BT_Chn7_Min, BT_Chn7_StdDev_Tmpy)
				  
  CALL Destroy_Spatial_Uniformity(Refl_Chn2_Mean, Refl_Chn2_Max, &
                                  Refl_Chn2_Min, Refl_Chn2_StdDev_Tmpy)
  
  !---------------------------------------------------------------
  ! Nullify pointers created in this routine.
  !---------------------------------------------------------------
 
  BT_Chn14_Msk => NULL()
  BT_Chn7_Msk => NULL()
  Refl_Chn2_Msk => NULL()
  
  Xgrad_Emiss_Chn14 => NULL()
  Ygrad_Emiss_Chn14 => NULL()
  Xgrad_Rev_Emiss_Chn14 => NULL()
  Ygrad_Rev_Emiss_Chn14 => NULL()
  
  Xgrad_Emiss_Chn7 => NULL()
  Ygrad_Emiss_Chn7 => NULL()
  Xgrad_Rev_Emiss_Chn7 => NULL()
  Ygrad_Rev_Emiss_Chn7 => NULL()
  
  Xgrad_Refl_Chn2 => NULL()
  Ygrad_Refl_Chn2 => NULL()
  Xgrad_Rev_Refl_Chn2 => NULL()
  Ygrad_Rev_Refl_Chn2 => NULL()
  Nsteps_Grad_Rev_Refl_Chn2 => NULL()
  
  Emiss_Chn7_Tot => NULL()
  Emiss_Chn14_Tot => NULL()
  Emiss_Chn14_Tot_Lrc => NULL()
  
  Emiss_Chn7_Ratio => NULL()
  Emiss_Chn7_Atm_Corrected => NULL()
  
  BT_Chn14_StdDev => NULL()
  BT_Chn14_StdDev_Lrc => NULL()
  BT_Chn14_Cld_Edge => NULL()
  
  BT_Chn7_StdDev => NULL()
  BT_Chn7_StdDev_Lrc => NULL()
  BT_Chn7_Cld_Edge => NULL()
  
  Refl_Chn2_StdDev => NULL()
  Refl_Chn2_StdDev_Lrc => NULL()
  Refl_Chn2_Cld_Edge => NULL()
  Refl_Chn2_Cld => NULL()
  Refl_Chn2_Diff => NULL()
  
  Tsfc_Obs => NULL()
  Tsfc_Bias => NULL()
  
  Fog_Prob_Refl_Chn2_Tsfc_Bias => NULL()
  Fog_Prob_Refl_Chn2_BT_Chn14_Diff => NULL()
  Fog_Prob_Emiss_Chn14_Tsfc_Bias => NULL()
  Fog_Prob_Emiss_Chn14_BT_Chn14_Diff => NULL()
  Fog_Prob_Emiss_Chn7_Tsfc_Bias => NULL()
  Fog_Prob_Emiss_Chn7_BT_Chn14_Diff => NULL()
  Fog_Prob => NULL()
  Fog_Depth => NULL()
  
  !---------------------------------------------------------------
  ! Deallocate LUT table
  !---------------------------------------------------------------

  IF (Sat%Iseg == (Sat%Nseg-1)) THEN
    
    CALL Display_Message(Routine_Name, &
                         "Deallocating fog LUT's", &
		         Sym%INFORMATION)
    
    DEALLOCATE(Refl_Chn2_Bin_Start_LUT, &
               Refl_Chn2_Bin_End_LUT, &
	       Emiss_Chn14_Bin_Start_LUT, &
	       Emiss_Chn14_Bin_End_LUT, &
	       Emiss_Chn7_Bin_Start_LUT, &
	       Emiss_Chn7_Bin_End_LUT, &
               Fog_Probability_Refl_Chn2_Tsfc_Bias_Water, &
               Fog_Probability_Refl_Chn2_Tsfc_Bias_Land, &
	       Fog_Probability_Refl_Chn2_BT_Chn14_Diff_Land, &
	       Fog_Probability_Emiss_Chn14_Tsfc_Bias_Water, &
               Fog_Probability_Emiss_Chn14_Tsfc_Bias_Land, &
	       Fog_Probability_Emiss_Chn14_BT_Chn14_Diff_Land, &
	       Fog_Probability_Emiss_Chn7_Tsfc_Bias_Water, &
               Fog_Probability_Emiss_Chn7_Tsfc_Bias_Land, &
	       Fog_Probability_Emiss_Chn7_BT_Chn14_Diff_Land, Stat=Astatus)
    
    IF (Astatus /= 0) THEN
      CALL Display_Message(Routine_Name, &
                           "Error deallocating fog probability tables", &
		           Sym%FAILURE)
      STOP
    ENDIF
  ENDIF
  
END SUBROUTINE Fog_Detection_Main

!-----------------------------------------------------------------
! Subroutine used to read in fog probability look-up table (LUT)
!-----------------------------------------------------------------

SUBROUTINE Read_Fog_LUT(Path, Filename)

  CHARACTER(Len=*), PARAMETER::Routine_Name = &
    "FOG_DETECTION:read_fog_lut"
  
  CHARACTER(Len=*), INTENT(IN) :: Path
  CHARACTER(Len=*), INTENT(IN) :: Filename
  
  CHARACTER(Len=256) :: Path_And_Filename
  INTEGER(KIND=Int4) :: Status
  INTEGER(KIND=Int4) :: Id
  INTEGER(KIND=Int4), DIMENSION(1) :: Start_1D
  INTEGER(KIND=Int4), DIMENSION(1) :: Stride_1D
  INTEGER(KIND=Int4), DIMENSION(1) :: Edge_1D
  INTEGER(KIND=Int4), DIMENSION(3) :: Start_3D
  INTEGER(KIND=Int4), DIMENSION(3) :: Stride_3D
  INTEGER(KIND=Int4), DIMENSION(3) :: Edge_3D
  
  Path_And_Filename = Trim(Path)//"/"//Trim(Filename)
  
  Status = Open_File_Hdf_Read(Trim(Path_And_Filename),Id)
  IF (Status == Sym%FAILURE) THEN
    PRINT "(/,a,'Failed to open, ',a)",EXE_PROMPT,Trim(Path_And_Filename)
    STOP
  ENDIF
  
  Status = Hdf_Sds_Attribute_Reader(Id, "", "NBINS_REF2", Refl_Chn2_Nbins_LUT)
  Status = Hdf_Sds_Attribute_Reader(Id, "", "NBINS_EMISS14", Emiss_Chn14_Nbins_LUT)
  Status = Hdf_Sds_Attribute_Reader(Id, "", "NBINS_EMS7_AC", Emiss_Chn7_Nbins_LUT)
  
  Status = Hdf_Sds_Attribute_Reader(Id, "", "NBINS_TSFC_BIAS", Tsfc_Bias_Nbins_LUT)
  Status = Hdf_Sds_Attribute_Reader(Id, "", "NBINS_BT14_DIFF", BT_Chn14_Diff_Nbins_LUT)
  Status = Hdf_Sds_Attribute_Reader(Id, "", "NBINS_REF2_STDDEV", Refl_Chn2_StdDev_Nbins_LUT)
  Status = Hdf_Sds_Attribute_Reader(Id, "", "NBINS_BT14_STDDEV", BT_Chn14_StdDev_Nbins_LUT)
  
  Status = Hdf_Sds_Attribute_Reader(Id, "", "MIN_TSFC_BIAS", Tsfc_Bias_Min_LUT)
  Status = Hdf_Sds_Attribute_Reader(Id, "", "MIN_BT14_DIFF", BT_Chn14_Diff_Min_LUT)
  Status = Hdf_Sds_Attribute_Reader(Id, "", "MIN_REF2_STDDEV", Refl_Chn2_StdDev_Min_LUT)
  Status = Hdf_Sds_Attribute_Reader(Id, "", "MIN_BT14_STDDEV", BT_Chn14_StdDev_Min_LUT)
  
  Status = Hdf_Sds_Attribute_Reader(Id, "", "MAX_TSFC_BIAS", Tsfc_Bias_Max_LUT)
  Status = Hdf_Sds_Attribute_Reader(Id, "", "MAX_BT14_DIFF", BT_Chn14_Diff_Max_LUT)
  Status = Hdf_Sds_Attribute_Reader(Id, "", "MAX_REF2_STDDEV", Refl_Chn2_StdDev_Max_LUT)
  Status = Hdf_Sds_Attribute_Reader(Id, "", "MAX_BT14_STDDEV", BT_Chn14_StdDev_Max_LUT)
  
  Status = Hdf_Sds_Attribute_Reader(Id, "", "DBIN_TSFC_BIAS", Tsfc_Bias_Increment_LUT)
  Status = Hdf_Sds_Attribute_Reader(Id, "", "DBIN_BT14_DIFF", BT_Chn14_Diff_Increment_LUT)
  Status = Hdf_Sds_Attribute_Reader(Id, "", "DBIN_REF2_STDDEV", Refl_Chn2_StdDev_Increment_LUT)
  Status = Hdf_Sds_Attribute_Reader(Id, "", "DBIN_BT14_STDDEV", BT_Chn14_StdDev_Increment_LUT)
  
  Stride_3D = (/1,1,1/)
  Start_3D = (/0,0,0/)
  
  Edge_3D = (/999999,999999,999999/)
  Status = Hdf_Sds_Reader(Id, "FOG_PROBABILITY_REF2_TSFC_BIAS_WATER", Start_3D, Stride_3D, Edge_3D, Fog_Probability_Refl_Chn2_Tsfc_Bias_Water)
  
  Edge_3D = (/999999,999999,999999/)
  Status = Hdf_Sds_Reader(Id, "FOG_PROBABILITY_REF2_TSFC_BIAS_LAND", Start_3D, Stride_3D, Edge_3D, Fog_Probability_Refl_Chn2_Tsfc_Bias_Land)
  
  Edge_3D = (/999999,999999,999999/)
  Status = Hdf_Sds_Reader(Id, "FOG_PROBABILITY_REF2_BT14_DIFF_LAND", Start_3D, Stride_3D, Edge_3D, Fog_Probability_Refl_Chn2_BT_Chn14_Diff_Land)
  
  Edge_3D = (/999999,999999,999999/)
  Status = Hdf_Sds_Reader(Id, "FOG_PROBABILITY_EMS7_TSFC_BIAS_WATER", Start_3D, Stride_3D, Edge_3D, Fog_Probability_Emiss_Chn7_Tsfc_Bias_Water)
  
  Edge_3D = (/999999,999999,999999/)
  Status = Hdf_Sds_Reader(Id, "FOG_PROBABILITY_EMS7_TSFC_BIAS_LAND", Start_3D, Stride_3D, Edge_3D, Fog_Probability_Emiss_Chn7_Tsfc_Bias_Land)
  
  Edge_3D = (/999999,999999,999999/)
  Status = Hdf_Sds_Reader(Id, "FOG_PROBABILITY_EMS7_BT14_DIFF_LAND", Start_3D, Stride_3D, Edge_3D, Fog_Probability_Emiss_Chn7_BT_Chn14_Diff_Land)
  
  Edge_3D = (/999999,999999,999999/)
  Status = Hdf_Sds_Reader(Id, "FOG_PROBABILITY_EMISS14_TSFC_BIAS_WATER", Start_3D, Stride_3D, Edge_3D, Fog_Probability_Emiss_Chn14_Tsfc_Bias_Water)
  
  Edge_3D = (/999999,999999,999999/)
  Status = Hdf_Sds_Reader(Id, "FOG_PROBABILITY_EMISS14_TSFC_BIAS_LAND", Start_3D, Stride_3D, Edge_3D, Fog_Probability_Emiss_Chn14_Tsfc_Bias_Land)
  
  Edge_3D = (/999999,999999,999999/)
  Status = Hdf_Sds_Reader(Id, "FOG_PROBABILITY_EMISS14_BT14_DIFF_LAND", Start_3D, Stride_3D, Edge_3D, Fog_Probability_Emiss_Chn14_BT_Chn14_Diff_Land)
  
  Start_1D = (/0/)
  Stride_1D = (/1/)
  
  Edge_1D =(/999999/)
  Status = Hdf_Sds_Reader(Id,"EMS7_AC_LOW", Start_1D, Stride_1D, Edge_1D, Emiss_Chn7_Bin_Start_LUT)
  
  Edge_1D =(/999999/)
  Status = Hdf_Sds_Reader(Id,"EMS7_AC_HIGH", Start_1D, Stride_1D, Edge_1D, Emiss_Chn7_Bin_End_LUT)
  
  Edge_1D =(/999999/)
  Status = Hdf_Sds_Reader(Id,"REF2_LOW", Start_1D, Stride_1D, Edge_1D, Refl_Chn2_Bin_Start_LUT)
  
  Edge_1D =(/999999/)
  Status = Hdf_Sds_Reader(Id,"REF2_HIGH", Start_1D, Stride_1D, Edge_1D, Refl_Chn2_Bin_End_LUT)
  
  Edge_1D =(/999999/)
  Status = Hdf_Sds_Reader(Id,"EMISS14_LOW", Start_1D, Stride_1D, Edge_1D, Emiss_Chn14_Bin_Start_LUT)
  
  Edge_1D =(/999999/)
  Status = Hdf_Sds_Reader(Id,"EMISS14_HIGH", Start_1D, Stride_1D, Edge_1D, Emiss_Chn14_Bin_End_LUT)
  
  IF (Status == Sym%FAILURE) THEN
    
    CALL Display_Message(Routine_Name, &
                         "Error reading fog probability tables", &
		         Sym%FAILURE)
    STOP
    
  ENDIF
  
  CALL Close_File_Hdf_Read(Id,Trim(Path_And_Filename))
  
END SUBROUTINE Read_Fog_LUT

END MODULE Fog_Detection
